<?php

/**
 * Description of Model_Customers
 *
 * @author
 */
use RedBean_Facade as R;
use Helpers\PasswordHelper as PasswordHelper;
use Enumerations\HttpStatusCode as HttpStatusCode;
use Enumerations\AccountType as AccountType;
use Enumerations\StatusType as StatusType;
use Enumerations\ErrorCodeType as ErrorCodeType;
use Exceptions\ApiException as ApiException;
use Valitron\Validator as Validator;
use Helpers\RedBeanHelper as RedBeanHelper;
//Require needed models

class Model_Customers extends RedBean_SimpleModel implements ModelBaseInterface {

    /**
     * Identifier
     * @var int
     */
    public $id;

    /**
     * User email
     * @var string
     */
    public $Email;
	
	

    /**
     * Constructor
     */
    public function __construct() {

    }

    /**
     * Check the email and password
    */
    public static function checkLogin($email,$password,$store){
		$result	= array();
		if($email =='' && $password =='') {
			// the User credentials was not found
           	throw new ApiException("Your email or password was incorrect.", ErrorCodeType::SomeFieldsMissing);
		}
		if($email != '' && $password != ''){
			if($store == 1){
				$result	 = R::findOne('users', 'Email = ? and Password = ? and Status = ?', array($email,$password,StatusType::ActiveStatus));
			}else if($store == 2){
				$result	 = R::findOne('admins', 'UserName = ? and Password = ? and IsSubAdmin = ? and Status = ?', array($email,$password,1,StatusType::ActiveStatus));
			}else if($store == 3){
				$result	 = R::findOne('admins', 'UserName = ? and Password = ? and IsSubAdmin = ? and Status = ?', array($email,$password,0,StatusType::ActiveStatus));
			}else{
				$result	 = R::findOne('customers', 'Email = ? and Password = ? and Status = ?', array($email,PasswordHelper::encrypt($password),StatusType::ActiveStatus));
			}
		}
		if (!$result) {
            return false;
        }else{
			if($store == 1){
				$users 					= R::dispense('users');
				$users->LastLoginDate 	= date('Y-m-d H:i:s');
				$users->id 				= $result['id'];
				R::Store($users);
			}else if($store == 2 || $store == 3){
				$admins 					= R::dispense('admins');
				$admins->LastLoginDate 		= date('Y-m-d H:i:s');
				$admins->id 				= $result['id'];
				R::Store($admins);
			}else{
				$customer 				 = R::dispense('customers');
				$customer->LastLoginDate = date('Y-m-d H:i:s');
				$customer->id 			 = $result['id'];
				R::Store($customer);
			}
			$result['Store']	=	$store;
			return $result;
		}
    }
	/**
     * Create an user account
	 * Validation for email , fbId,TwitterId,GooglePlusId,Username
     */
    public function create(){ // receiptmatch creating
		 /**
         * Get the bean
         * @var $bean Model_Customers
         */
        $bean 		= $this->bean;
		
		// validate the model
        $this->validate();

        // validate the creation
        $this->validateCreate();
		
        $bean->DateCreated 			= date('Y-m-d H:i:s');
        $bean->DateModified 		= $bean->DateCreated;
		
		// encrypt the password
		if($bean->Password){
			$bean->ActualPassword  	= $bean->Password ;
        	$bean->Password 		= PasswordHelper::encrypt($bean->Password);
		}
		$bean->Status 				= StatusType::OnStatus;
	   // save the bean to the database
		$customerId 	=	 R::store($this);
	    return $customerId;
    }

	public function validate(){ // beacon creating
		$bean = $this->bean;
		$rules = [
            'required' => [
                ['FirstName'],['LastName'],['Email'],['Password']
            ],
			'Email' => 'Email',
        ];
		$v = new Validator($this->bean);
        $v->rules($rules);
        if (!$v->validate()) {
            $errors = $v->errors();
            throw new ApiException("Please check the user's properties" ,  ErrorCodeType::SomeFieldsMissing, $errors);
        }
    }
	/**
     * Validate the creation of an account
     * @throws ApiException if the user being creating the account with already exists of email in the database.
     */
    public function validateCreate(){

        /**
         * Get the bean
         * @var $bean Model_Customers
         */
        $bean = $this->bean;
		/**
         * Email Id must be unique
         */
		 if($bean->Email != '') {
	        $existingAccount = R::findOne('customers', 'Email = ? and Status <> ? order by DateModified desc', array($bean->Email,StatusType::DeleteStatus));
	        if ($existingAccount) {
				// an account with that email already exists in the system - don't create account
          		 throw new ApiException("This email address is already associated with another account.", ErrorCodeType::EmailAlreadyExists);
			}
		}
    }
	/**
	* @param get city list
	*/
	public function getCityList($type=''){
	
		$returnArray = array();
		 /**
         * Get the bean
         * @var $bean Model_Customers
         */
		$cities		=	array();
		$sql		=	"select * from cities where Status = 1 order by CityName";
		$cities		= 	R::getAll($sql);
		if(!$cities){
			if($type != '1')
				throw new ApiException("No cities found.", ErrorCodeType::NoResultFound);
		}
		return $cities;
	}
	/**
	* @param get user types
	*/
	public function getUserTypes($type=''){
	
		$returnArray = array();
		 /**
         * Get the bean
         * @var $bean Model_Customers
         */
		$userTypes		=	array();
		$sql			=	"select * from usertypes where Status = 1 order by id asc";
		$userTypes		= 	R::getAll($sql);
		if(!$userTypes){
			if($type != '1')
				throw new ApiException("No usertypes found.", ErrorCodeType::NoResultFound);
		}
		return $userTypes;
	}
	/**
     * Customer tracking
	 * 
     */
    public function customerTracking(){ 
		 /**
         * Get the bean
         * @var $bean Model_Customers
         */
		// validate the creation
        $this->validateTracking();
		
		$this->validateTrackingParams();
		
        $bean 					= $this->bean;
		$tracking				= R::dispense('customertracking');
        $tracking->LoginDate 	= date('Y-m-d H:i:s');
        $tracking->fkUserTypeId = $bean->UserTypeId;
		$tracking->fkCityId 	= $bean->CityId;
	   // save the bean to the database
		$trackingId 			= R::store($tracking);
	    return $trackingId;
    }
	/**
     * Validate Customer tracking
	 * 
     */
	public function validateTracking(){ // beacon creating
		$bean = $this->bean;
		$rules = [
            'required' => [
                ['UserTypeId'],['CityId']
            ]
        ];
		$v = new Validator($this->bean);
        $v->rules($rules);
        if (!$v->validate()) {
            $errors = $v->errors();
            throw new ApiException("Please check the required properties" ,  ErrorCodeType::SomeFieldsMissing, $errors);
        }
    }
	/**
     * Validate the creation of tracking
     * @throws ApiException if the user being creating the account with unknown city and usertype.
     */
    public function validateTrackingParams(){

        /**
         * Get the bean
         * @var $bean Model_Customers
         */
        $bean = $this->bean;
		/**
         * check city and user type
         */
		 if($bean->UserTypeId != '') {
	        $existingAccount = R::findOne('usertypes', 'id = ? and Status = ?', array($bean->UserTypeId,StatusType::ActiveStatus));
	        if(!$existingAccount) {
				// user type not exist
          		 throw new ApiException("Invalid user type.", ErrorCodeType::NoResultFound);
			}
		}
		if($bean->CityId != '') {
	        $existingAccount = R::findOne('cities', 'id = ? and Status = ?', array($bean->CityId,StatusType::ActiveStatus));
	        if(!$existingAccount) {
				// city not exist
          		 throw new ApiException("Invalid city.", ErrorCodeType::NoResultFound);
			}
		}
    }
	/**
	* @param get city list and user types
	*/
	public function getCityUserTypesList(){
	
		$returnArray 	= $userTypesArray	= $cityArray =  array();
		
		$userTypesArray	= $this->getUserTypes(1);
		$cityArray		= $this->getCityList(1);
		$returnArray['cities']		=	$cityArray;
		$returnArray['usertypes']	=	$userTypesArray;
		return $returnArray;
	}
}
