<?php

/**
 * @author 
 */

/**
 * Load configuration
 */
require_once('../../config.php');

/**
 * Load libraries
 */
require('../../vendor/autoload.php');                   // composer library
require_once '../../lib/beaconApi.php';               	// application
require_once '../../lib/Helpers/PasswordHelper.php';    // password helper
require_once '../../lib/beaconApiResponse.php';       	// response
require_once '../../lib/beaconApiResponseMeta.php';   	// response meta
require_once '../../lib/ModelBaseInterface.php';        // base interface class for RedBean models
require_once '../../lib/Model_Customers_v3.php';        // Model: Users

require_once "../../includes/CommonFunctions.php";

use Helpers\ResponseHelper as ResponseHelper;
/**
 * Initialize application
 *
 * We set $startAuthServer to true to start the authorization server
 */
beaconApi::init(true);
$app = new \Slim\Slim();

/**
 * Check the Facebook and linkedIn Id callback function
 */
$checkLoginCallBack = function($email,$password,$store) {
		return Model_Customers::checkLogin($email,$password,$store);
};
 
/**
 *
 * This is how you obtain an authentication token
 * We are doing a post to the following endpoint: /oauth2/password/token
 * When doing so we pass the following parameters:
 
 * - ClientId
 * - ClientSecret
 * - UserName
 * - Password
 *
 * path: /oauth2/password/token
 */
$app->post('/token', function () use ($app, $checkLoginCallBack) {
	try {

        $req = $app->request();
		$res = $app->response();
		$res['Content-Type'] = 'application/json';

        // grab the authorization server from the api
        $authServer = beaconApi::$authServer;
        // We are going for this flow in oauth 2.0: Resource Owner Password Credentials Grant
        $grant = new League\OAuth2\Server\Grant\Password($authServer);

        // this is where we check the Facebook and linkedIn Id
		
		$user_id = $grant->setVerifyCredentialsCallback($checkLoginCallBack);
        $authServer->addGrantType($grant);
        // get the response from the server
        $response = $authServer->getGrantType('password')->completeFlow();
		/** TO ASSIGN THE RESPONSES TO A FUNCTION **/
		ResponseHelper::setResponse(json_encode($response));
		/** END TO ASSIGN THE RESPONSES TO A FUNCTION **/
        echo json_encode($response);

    }

    catch (League\OAuth2\Server\Exception\ClientException $e) {

        // Get the http status code based on the oauth error
        $status = beaconApi::$errorCodeLookup[$e->getCode()];
		beaconApi::showError($e, $status);
    }
    catch (Exception $e) {

        // Something went wrong
       /* beaconApi::showError($e);*/
		/*beaconApi::showError(
            $e,
            $e->getHttpStatusCode(),
            $e->getErrors()
        );*/
		$status = beaconApi::$errorCodeLookup[$e->getCode()];
		beaconApi::showError($e, $status);
    }
});

/**
 * Start the Slim Application
 */
$app->run();